/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qcursor.h>
#include <qpixmap.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kdebug.h>
#include "taviasidebar.h"
#include "tschtmlview.h"
#include "tscrssview.h"
#include "tsclirsview.h"


TaviaSidebar::TaviaSidebar(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
    init();
    config = new KConfig("taviasidebarrc",false,true);
    contentsInfo.setAutoDelete(true);
    containerList.setAutoDelete(true);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    QLabel *label = new QLabel(this,"label");
    label->setText(i18n("Sidebar"));
    label->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);
    QPushButton *b = new QPushButton(this,"newContents");
    b->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("filenew",KIcon::Small,KIcon::SizeSmall));
    connect(b,SIGNAL(clicked()),this,SLOT(ncMenuPopup()));
    dummy = new QLabel(this,"dummy");
    dummy->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    dummy->show();
    bodyLayout = new QVBoxLayout(this);
    QHBoxLayout *hlay = new QHBoxLayout(bodyLayout);
    hlay->addWidget(label);
    hlay->addWidget(b);
    bodyLayout->addWidget(dummy);
    //loadConfig();
}


TaviaSidebar::~TaviaSidebar()
{
    saveConfig();
    delete config;
}

/*!
    \fn TaviaSidebar::init()
 */
void TaviaSidebar::init()
{
    contentsInfo.append(TSCHTMLView::getContentsInfo());
    contentsInfo.append(TSCRSSView::getContentsInfo());
    contentsInfo.append(TSCLIRSView::getContentsInfo());


    newContentsPopup = new QPopupMenu(this,"newContentsPopup");
    for(unsigned int i = 0;i < contentsInfo.count();i ++) {
        newContentsPopup->insertItem(
            KGlobal::instance()->iconLoader()->loadIconSet(contentsInfo.at(i)->icon,KIcon::Small,KIcon::SizeSmall),
            contentsInfo.at(i)->name,i);
    }
    connect(newContentsPopup,SIGNAL(activated(int)),this,SLOT(createContents(int)));
}

/*!
    \fn TaviaSidebar::ncMenuPopup()
 */
void TaviaSidebar::ncMenuPopup()
{
    newContentsPopup->popup(QCursor::pos());
}

/*!
    \fn TaviaSidebar::createContents(int i,QString id)
 */
void TaviaSidebar::createContents(int i,QString id)
{
    if (id.isNull()) id = createUniqueId();
    // multipleʤϥ˥򺣸ԲĤ
    if (contentsInfo.at(i)->multiple == false) {
        newContentsPopup->setItemEnabled(i,false);
    }
//    TaviaSidebarContents *c = contentsInfo.at(i)->createContents(id);
//    c->setUniqueId(id);
    TaviaSidebarContainer *t = new TaviaSidebarContainer(this,contentsInfo.at(i)->createContents(id));
    t->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    t->show();
    bodyLayout->addWidget(t);
    t->type = i;
    connect(t,SIGNAL(buttonClicked(TaviaSidebarContainer*)),this,SLOT(changeActiveContents(TaviaSidebarContainer*)));
    connect(t,SIGNAL(closeContents(TaviaSidebarContainer *)),this,SLOT(deleteContents(TaviaSidebarContainer *)));
    // ɬSIGNAL餤󤷡᤮
    connect(t->getContents(),SIGNAL(openURL(const KURL &)),this,SIGNAL(openURL(const KURL &)));
    connect(t->getContents(),SIGNAL(openURL(const KURL &,const KParts::URLArgs &)),this,SIGNAL(openURL(const KURL &,const KParts::URLArgs &)));
    connect(t->getContents(),SIGNAL(openURLwithNewTab(const KURL &)),this,SIGNAL(openURLwithNewTab(const KURL &)));
    connect(t->getContents(),SIGNAL(openURLwithNewTab(const KURL &,const KParts::URLArgs &)),this,SIGNAL(openURLwithNewTab(const KURL &,const KParts::URLArgs &)));

    emit changeActiveContents(t);
    containerList.append(t);
}

/*!
    \fn TaviaSidebar::changeActiveContents(TaviaSidebarContainer *c)
 */
void TaviaSidebar::changeActiveContents(TaviaSidebarContainer *c)
{
    if (c != 0) {
        dummy->hide();
    } else {
        dummy->show();
    }
    for(unsigned int i = 0;i < containerList.count();i ++) {
        if (containerList.at(i) == c) {
            c->showContents();
        } else {
            containerList.at(i)->hideContents();
        }
    }
}

/*!
    \fn TaviaSidebar::deleteContents(TaviaSidebarContainer *c)
 */
void TaviaSidebar::deleteContents(TaviaSidebarContainer *c)
{
    // mulitpleʤν
    if (contentsInfo.at(c->type)->multiple == false) {
        newContentsPopup->setItemEnabled(c->type,true);
    }
    for(unsigned int i = 0;i < containerList.count();i ++) {
        if (c == containerList.at(i)) {
            //delete c;
            containerList.remove(i);
            if (containerList.count() == 0) {
                changeActiveContents(0);
            } else if (containerList.count() == i) {
                changeActiveContents(containerList.at(i - 1));
            } else {
                changeActiveContents(containerList.at(i));
            }
            break;
        }
    }
}

/*!
    \fn TaviaSidebar::createUniqueId()
 */
QString TaviaSidebar::createUniqueId()
{

    QString i;
    i =
        QString::number((QDate::fromString("1970-01-01",Qt::ISODate).daysTo(QDate::currentDate()) * 86400) +
        QTime::fromString("00:00:00",Qt::ISODate).secsTo(QTime::currentTime()))
    +
        QString::number(QTime::currentTime().msec());
    kdDebug() << "uniqId: " << i << endl;
    return(i);


}

/*!
    \fn TaviaSidebar::saveConfig()
 */
void TaviaSidebar::saveConfig()
{
    config->deleteGroup("Contents");
    config->setGroup("Contents");
    for(unsigned int i = 0;i < containerList.count();i ++) {
        QStringList l;
        l.append(containerList.at(i)->getContents()->getUniqueId());
        l.append(contentsInfo.at(
            (unsigned int)containerList.at(i)->type)->name);

        config->writeEntry(QString::number(i),l);
    }
}

/*!
    \fn TaviaSidebar::loadConfig()
 */
void TaviaSidebar::loadConfig()
{
    config->setGroup("Contents");
    int i = 0;
    while(-1) {
        QStringList l = config->readListEntry(QString::number(i));
        if (l.count() != 2) break;
        kdDebug() << "sidbarRestoreConfig: " << l[0] << "," << l[1] << endl;
        for(unsigned int j = 0;j < contentsInfo.count();j ++) {
            if (contentsInfo.at(j)->name == l[1]) {
                createContents((int)j,l[0]);
            }
        }
        i ++;
    }

}

/*!
    \fn TaviaSidebar::initSidebar()
 */
void TaviaSidebar::initSidebar()
{
    loadConfig();
}









#include "taviasidebar.moc"


