/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include <qlayout.h>
#include <qcursor.h>
#include <kglobal.h>
#include <kiconloader.h>
#include "taviasidebarcontainer.h"

TaviaSidebarContainerBar::TaviaSidebarContainerBar(QWidget * parent, const char * name)
 : QToolButton(parent, name)
{
}

TaviaSidebarContainerBar::~ TaviaSidebarContainerBar()
{
}

void TaviaSidebarContainerBar::mousePressEvent(QMouseEvent *e)
{
    if (e->button() == Qt::RightButton) emit rightButtonAction();
    else QToolButton::mousePressEvent(e);
}

TaviaSidebarContainer::TaviaSidebarContainer(QWidget *parent, TaviaSidebarContents *c, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    c->reparent(this,QPoint(),true);
    connect(c,SIGNAL(update()),this,SLOT(update()));
    labelButton = new TaviaSidebarContainerBar(this,"labelButton");
    labelButton->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    labelButton->setUsesTextLabel(true);
    labelButton->setTextPosition(QToolButton::Right);
    connect(labelButton,SIGNAL(clicked()),this,SLOT(clicked()));
    c->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

    contents = c;
    QVBoxLayout *vlay = new QVBoxLayout(this);
    vlay->addWidget(labelButton);
    vlay->addWidget(c);

    popup = new QPopupMenu(this,"popup");
    popup->insertItem(KGlobal::instance()->iconLoader()->loadIconSet("configure",KIcon::Small,KIcon::SizeSmall),
        i18n("configure"),c,SLOT(configure()));
    popup->setItemEnabled(
        popup->insertItem(KGlobal::instance()->iconLoader()->loadIconSet("attach",KIcon::Small,KIcon::SizeSmall),
        i18n("clip")),false);
    popup->insertSeparator();
    popup->insertItem(KGlobal::instance()->iconLoader()->loadIconSet("fileclose",KIcon::Small,KIcon::SizeSmall),
        i18n("close"),this,SLOT(slotClose()));

    labelButton->setPopup(popup);
    connect(labelButton,SIGNAL(rightButtonAction()),this,SLOT(menuPopup()));
    emit update();
}


TaviaSidebarContainer::~TaviaSidebarContainer()
{
    delete contents;
}

/*!
    \fn TaviaSidebarContainer::showContents()
 */
void TaviaSidebarContainer::showContents()
{
    contents->show();
}

/*!
    \fn TaviaSidebarContainer::hideContents()
 */
void TaviaSidebarContainer::hideContents()
{
    contents->hide();
}

/*!
    \fn TaviaSidebarContainer::clicked()
 */
void TaviaSidebarContainer::clicked()
{
    emit buttonClicked(this);
}

/*!
    \fn TaviaSidebarContainer::isContentsShown()
 */
bool TaviaSidebarContainer::isContentsShown()
{
    return(contents->isShown());
}

/*!
    \fn TaviaSidebarContainer::slotClose()
 */
void TaviaSidebarContainer::slotClose()
{
    contents->closeContents();
    emit closeContents(this);
}

/*!
    \fn TaviaSidebarContainer::update()
 */
void TaviaSidebarContainer::update()
{
    labelButton->setTextLabel(contents->getContentsTitle(),true);
    labelButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet(contents->getContentsIcon(),KIcon::Small,KIcon::SizeSmall));
}

/*!
    \fn TaviaSidebarContainer::getContents()
 */
TaviaSidebarContents * TaviaSidebarContainer::getContents()
{
    return(contents);
}

/*!
    \fn TaviaSidebarContainer::menuPopup()
 */
void TaviaSidebarContainer::menuPopup()
{
    popup->popup(QCursor::pos());
}










#include "taviasidebarcontainer.moc"

