/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "tschtmlview.h"
#include <qlabel.h>
#include <qpushbutton.h>

TSCHTMLView::TSCHTMLView(const QString id,QWidget *parent, const char *name, WFlags f)
 : TaviaSidebarContents(id,parent, name, f)
{
    isClose = false;
    uid = id;
    config = new KConfig("taviasidebarhtmlviewrc",false,true);
    createConfigPanel();
    loadConfig();
    khtml = new KHTMLPart(this);
    khtml->setJScriptEnabled(false);
    khtml->setJavaEnabled(false);
    khtml->setPluginsEnabled(false);
    khtml->setMetaRefreshEnabled(false);
    khtml->view()->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    QVBoxLayout *v = new QVBoxLayout(this);
    v->addWidget(configPanel);
    v->addWidget(khtml->view());
    khtml->openURL(KURL(url));
    connect(khtml->browserExtension(),SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)),
        this,SIGNAL(openURL(const KURL &,const KParts::URLArgs &)));
    connect(khtml,SIGNAL(completed()),SLOT(getTitleFromHtml()));
}


TSCHTMLView::~TSCHTMLView()
{
    if (!isClose) saveConfig();
    delete config;
}

/*!
    \fn TSCHTMLView::createContents(const QString id)
 */
TaviaSidebarContents * TSCHTMLView::createContents(const QString id)
{
    return(new TSCHTMLView(id));
}

/*!
    \fn TSCHTMLView::getContentsInfo()
 */
TaviaSidebarContentsInfo * TSCHTMLView::getContentsInfo()
{
    TaviaSidebarContentsInfo *t = new TaviaSidebarContentsInfo();
    t->multiple = true;
    t->name = "HTMLView";
    t->icon = "html";
    t->createContents = TSCHTMLView::createContents;
    return(t);
}

/*!
    \fn TSCHTMLView::getContentsTitle()
 */
const QString TSCHTMLView::getContentsTitle()
{
    return(title);
}

/*!
    \fn TSCHTMLView::getContentsIcon()
 */
const QString TSCHTMLView::getContentsIcon()
{
    return("html");
}

/*!
    \fn TSCHTMLView::loadConfig()
 */
void TSCHTMLView::loadConfig()
{
    //kdDebug() << getUniqueId() << " : " << config->readEntry("URL") <<endl;
    config->setGroup(uid);
    url = config->readEntry("URL","about:blank");
    title = config->readEntry("title","HTMLView");
    gtfhtml = config->readBoolEntry("titlefromhtml",false);
}

/*!
    \fn TSCHTMLView::saveConfig()
 */
void TSCHTMLView::saveConfig()
{
    config->setGroup(uid);
    config->writeEntry("URL",url);
    config->writeEntry("title",title);
    config->writeEntry("titlefromhtml",gtfhtml);
}

/*!
    \fn TSCHTMLView::closeContents()
 */
void TSCHTMLView::closeContents()
{
    config->deleteGroup(getUniqueId());
    isClose = true;
}

/*!
    \fn TSCHTMLView::createConfigPanel()
 */
void TSCHTMLView::createConfigPanel()
{
    configPanel = new QFrame(this);
    configPanel->hide();
    QLabel *urllabel = new QLabel(configPanel);
    urllabel->setText(i18n("URL:"));
    curl = new KLineEdit(configPanel);
    QLabel *titlelabel = new QLabel(configPanel);
    titlelabel->setText(i18n("title:"));
    ctitle = new KLineEdit(configPanel);
    cgtfhtml = new QCheckBox(configPanel);
    cgtfhtml->setText(i18n("get title from HTML"));
    connect(cgtfhtml,SIGNAL(toggled(bool)),ctitle,SLOT(setDisabled(bool)));
    QPushButton *ok = new QPushButton(configPanel);
    ok->setText(i18n("ok"));
    QPushButton *cancel = new QPushButton(configPanel);
    cancel->setText(i18n("cancel"));
    connect(ok,SIGNAL(clicked()),this,SLOT(configOk()));
    connect(cancel,SIGNAL(clicked()),this,SLOT(configCancel()));

    QVBoxLayout *v = new QVBoxLayout(configPanel);
    QHBoxLayout *h1 = new QHBoxLayout(v);
    h1->addWidget(urllabel);
    h1->addWidget(curl);
    v->addLayout(h1);
    QHBoxLayout *h2 = new QHBoxLayout(v);
    h2->addWidget(titlelabel);
    h2->addWidget(ctitle);
    v->addLayout(h2);
    v->addWidget(cgtfhtml);
    QHBoxLayout *h3 = new QHBoxLayout(v);
    h3->addWidget(ok);
    h3->addWidget(cancel);

}

/*!
    \fn TSCHTMLView::configure()
 */
void TSCHTMLView::configure()
{
    if (!isShown()) return;
    curl->setText(url);
    ctitle->setText(title);
    cgtfhtml->setChecked(gtfhtml);
    if (cgtfhtml->isChecked()) emit ctitle->setEnabled(false);

    configPanel->show();
}

/*!
    \fn TSCHTMLView::configOk()
 */
void TSCHTMLView::configOk()
{
    configPanel->hide();
    url = curl->text();
    title = ctitle->text();
    gtfhtml = cgtfhtml->isChecked();
    saveConfig();
    khtml->openURL(KURL(url));
    emit update();
}

/*!
    \fn TSCHTMLView::configCancel()
 */
void TSCHTMLView::configCancel()
{
    configPanel->hide();
}

/*!
    \fn TSCHTMLView::getTitleFromHtml()
 */
void TSCHTMLView::getTitleFromHtml()
{
    if (gtfhtml) {
        title = khtml->htmlDocument().title().string().replace(QChar('\n')," ");
        emit update();
    }
}










#include "tschtmlview.moc"
