/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "tsclirsview.h"
#include <klocale.h>
#include <kdebug.h>
#include <qstringlist.h>
#include <qdatetime.h>
#include <qfile.h>
#include <qtextcodec.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qlabel.h>
#include <qlistview.h>
#include <kio/netaccess.h>


TSCLIRSView::TSCLIRSView(const QString id, QWidget *parent, const char *name, WFlags f)
 : TaviaSidebarContents(id, parent, name, f)
{
    config = new KConfig("taviasidebarlirsviewrc",false,true);
    entry.setAutoDelete(true);


    QVBoxLayout *vlay = new QVBoxLayout(this);
    createConfigPanel();
    listView = new KListView(this,"listView");
    listView->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    listView->addColumn(i18n("PageTitle"));
    listView->addColumn(i18n("Administrator"));
    listView->addColumn(i18n("Last-Modified"));
    listView->addColumn(i18n("URL"));
    listView->setSelectionMode(QListView::Extended);
    listView->setSorting(2,false);
    entryMenu = new QPopupMenu(listView,"entryMenu");
    entryMenu->insertItem(i18n("open with newTab"),this,SLOT(openURLwnt()),0,0);
    connect(listView,SIGNAL(rightButtonPressed(QListViewItem *,const QPoint &,int)),
        this,SLOT(entryPopup(QListViewItem *,const QPoint &,int)));
    connect(listView,SIGNAL(doubleClicked(QListViewItem *,const QPoint&,int)),
        this,SLOT(itemDoubleClicked(QListViewItem*,const QPoint&,int)));
    QHBoxLayout *h = new QHBoxLayout(this);
    QSpacerItem *s = new QSpacerItem(1,1,QSizePolicy::Expanding,QSizePolicy::Minimum);
    reload = new QPushButton(this,"reload");
    reload->setText(i18n("reload"));
    connect(reload,SIGNAL(clicked()),this,SLOT(getListFiles()));
    h->addItem(s);
    h->addWidget(reload);
    vlay->addWidget(configPanel);
    vlay->addWidget(listView);
    vlay->addLayout(h);
    loadConfig();
    getListFiles();
    //openLocalList("/home/tamra/sites.lirs");
    //setListView();

}


TSCLIRSView::~TSCLIRSView()
{
    saveConfig();
}


/*!
    \fn TSCLIRSView::createContents(const QString id)
 */
TaviaSidebarContents * TSCLIRSView::createContents(const QString id)
{
    return(new TSCLIRSView(id));
}


/*!
    \fn TSCLIRSView::getContentsInfo()
 */
TaviaSidebarContentsInfo * TSCLIRSView::getContentsInfo()
{
    TaviaSidebarContentsInfo *t = new TaviaSidebarContentsInfo();
    t->multiple = false;
    t->name = "LIRSView";
    t->icon = "reload";
    t->createContents = TSCLIRSView::createContents;
    return(t);
}


/*!
    \fn TSCLIRSView::getContentsTitle()
 */
const QString TSCLIRSView::getContentsTitle()
{
    return(i18n("LIRS List"));
}


/*!
    \fn TSCLIRSView::getContentsIcon()
 */
const QString TSCLIRSView::getContentsIcon()
{
    return("reload");
}


/*!
    \fn TSCLIRSView::configure()
 */
void TSCLIRSView::configure()
{
    if (!isShown()) return;
    fileline->setText("");
    flist->clearSelection();
    configPanel->show();
}


/*!
    \fn TSCLIRSView::entryParse(QString line)
 */
LIRSEntry * TSCLIRSView::entryParse(QString line)
{
    // ܤĤ֤äƲ龯Ȥ򤷤ޤ
    line.replace("\\,","\\_");
    QStringList l = QStringList::split(QString(","),line);
    for (unsigned int i = 0;l.count() < i;i ++) {
        l[i].replace("\\_","\\,");
    }
    // ֤꤬Ȥܤ򳫤Ʋ
    if (l[0] != "LIRS") return(0);
    LIRSEntry *e = new LIRSEntry();
    e->lMod = l[1].toInt();
    e->lDet = l[2].toInt();
    e->td = l[3].toInt();
    e->cl = l[4].toInt();
    e->url = l[5];
    e->title = l[6];
    e->adm = l[7];
    e->from = l[8];
    kdDebug() << " lMod:" << e->lMod
        << " lDet:" << e->lDet
        << " td:" << e->td
        << " cl:" << e->cl
        << " url:" << e->url
        << " title:" << e->title
        << " adm:" << e->adm
        << " from:" << e->from << endl;
    QDateTime t;
    t.setTime_t(e->lMod,Qt::LocalTime);
    kdDebug() << "Modif: " << t.toString() << endl;
    return(e);
}

/*!
    \fn TSCLIRSView::openLocalList(const QString filename)
 */
int TSCLIRSView::openLocalList(const QString filename)
{
    if (!QFile::exists(filename)) return(-1);
    QFile list(filename);
    list.open(IO_ReadOnly);
    QTextCodec *codec = QTextCodec::codecForName("eucJP");
    char bf[255];
    int c = 0;
    while(-1) {
        if (list.readLine(bf,255) == -1) break;
        LIRSEntry* e = entryParse(codec->toUnicode(bf));
        if (e != 0) {
            // ʲʣ򥳡ɡʤ
            for(unsigned int i = 0;i < entry.count();i ++) {
                if (e->url == entry.at(i)->url) {
                    if (e->lMod > entry.at(i)->lMod) {
                        entry.remove(i);
                        break;
                    } else {
                        delete e;
                        e = 0;
                        break;
                    }
                }
            }
            if (e != 0) entry.append(e);
            c ++;
        }
    }
    list.close();
    return(c);
}




/*!
    \fn TSCLIRSView::setListView()
 */
void TSCLIRSView::setListView()
{
    listView->clear(); // QListViewItemdeleteⴰλʤΤɤǧ
    for(unsigned int i = 0;i < entry.count();i ++) {
        if (entry.at(i)->lMod != 0) {
            QDateTime t;
            t.setTime_t(entry.at(i)->lMod,Qt::LocalTime); // + entry.at(i)->td,Qt::LocalTime);
            QListViewItem *li = new QListViewItem(listView,entry.at(i)->title,
                entry.at(i)->adm,
                t.toString(QString("yyyy/MM/dd-hh:mm:ss")),
                entry.at(i)->url);
            listView->insertItem(li);
        }
    }
    reload->setEnabled(true);
}

/*!
    \fn TSCLIRSView::itemDoubleClicked(QListViewItem *item,const QPoint& p,int c)
 */
void TSCLIRSView::itemDoubleClicked(QListViewItem *item,const QPoint& p,int c)
{
    openURL(item->text(3));
}

/*!
    \fn TSCLIRSView::createConfigPanel()
 */
void TSCLIRSView::createConfigPanel()
{
    configPanel = new QFrame(this);
    configPanel->hide();
    QVBoxLayout *v = new QVBoxLayout(configPanel);
    flist = new KListView(configPanel,"flist");
    flist->addColumn(i18n("LIRS Filelist"));
    flist->setFixedHeight(100);
    flist->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    flist->setSelectionMode(QListView::Single);
    configMenu = new QPopupMenu(configPanel,"configMenu");
    configMenu->insertItem(i18n("add entry"),this,SLOT(configAddEntry()),0,0);
    configMenu->insertItem(i18n("delete this entry"),this,SLOT(configDelEntry()),0,1);
    connect(flist,SIGNAL(rightButtonPressed(QListViewItem *,const QPoint &,int)),
        this,SLOT(configPopup(QListViewItem *,const QPoint &,int)));
    connect(flist,SIGNAL(selectionChanged(QListViewItem *)),
        this,SLOT(configSelected(QListViewItem*)));
    v->addWidget(flist);
    QHBoxLayout *h1 = new QHBoxLayout(v);
    fileline = new KLineEdit(configPanel,"fileline");
    QPushButton *setButton = new QPushButton(configPanel,"set");
    setButton->setText(i18n("update"));
    connect(setButton,SIGNAL(clicked()),this,SLOT(configUpdate()));
    connect(fileline,SIGNAL(returnPressed(const QString &)),this,SLOT(configUpdate()));
    h1->addWidget(fileline);
    h1->addWidget(setButton);
    v->addLayout(h1);
    QPushButton *ok = new QPushButton(configPanel,"ok");
    connect(ok,SIGNAL(clicked()),this,SLOT(configOk()));
    ok->setText(i18n("ok"));
    v->addWidget(ok);
}

/*!
    \fn TSCLIRSView::configPopup(QListViewItem *i,const QPoint &p,int c)
 */
void TSCLIRSView::configPopup(QListViewItem *i,const QPoint &p,int c)
{
    if (i <= 0) {
        configMenu->setItemEnabled(1,false);
    } else {
        configMenu->setItemEnabled(1,true);
    }
    configMenu->popup(p);
}

/*!
    \fn TSCLIRSView::entryPopup(QListViewItem *i,const QPoint &p,int c)
 */
void TSCLIRSView::entryPopup(QListViewItem *i,const QPoint &p,int c)
{
    entryMenu->popup(p);
}

/*!
    \fn TSCLIRSView::configAddEntry()
 */
void TSCLIRSView::configAddEntry()
{
    flist->insertItem(new QListViewItem(flist,"LIRS URL"));
}


/*!
    \fn TSCLIRSView::configDelEntry()
 */
void TSCLIRSView::configDelEntry()
{
    if (flist->selectedItem() == 0) return;
    delete flist->selectedItem();
}

/*!
    \fn TSCLIRSView::configSelected(QListViewItem *item)
 */
void TSCLIRSView::configSelected(QListViewItem *item)
{
    fileline->setText(item->text(0));
}


/*!
    \fn TSCLIRSView::configUpdate()
 */
void TSCLIRSView::configUpdate()
{
    if (flist->selectedItem() == 0) return;
    if (fileline->text() == "") configDelEntry();
    flist->selectedItem()->setText(0,fileline->text());
}

/*!
    \fn TSCLIRSView::configOk()
 */
void TSCLIRSView::configOk()
{
    configPanel->hide();
    saveConfig();
    getListFiles();
}

/*!
    \fn TSCLIRSView::getListFiles()
 */
void TSCLIRSView::getListFiles()
{
    reload->setEnabled(false);
    entry.clear();
    QListViewItem *li;
    li = flist->firstChild();
    while(-1) {
        if (li == 0) break;
        kdDebug() << "FLIST: " << li->text(0) << endl;
        KURL src(li->text(0));
        if (!src.isLocalFile()) {
            QString tmp;
            if (KIO::NetAccess::download(src,tmp) != false) {
                openLocalList(tmp);
                KIO::NetAccess::removeTempFile(tmp);
            }
        } else {
            openLocalList(li->text(0));
        }
        li = li->nextSibling();
    }
    setListView();
}

/*!
    \fn TSCLIRSView::saveConfig()
 */
void TSCLIRSView::saveConfig()
{
    config->setGroup("LIRS");
    QStringList list;
    QListViewItem *li = flist->firstChild();
    while(-1) {
        if (li == 0) break;
        list.append(li->text(0));
        li = li->nextSibling();
    }
    config->writeEntry("files",list);
    config->sync();

}


/*!
    \fn TSCLIRSView::loadConfig()
 */
void TSCLIRSView::loadConfig()
{
    config->setGroup("LIRS");
    QStringList list = config->readListEntry("files");
    for(unsigned int i = 0;i < list.count();i ++) {
        flist->insertItem(new QListViewItem(flist,list[i]));
    }
}


/*!
    \fn TSCLIRSView::openURLwnt()
 */
void TSCLIRSView::openURLwnt()
{
    QPtrList<QListViewItem> p = listView->selectedItems();
    for(unsigned i = 0;i < p.count();i ++) {
        emit openURLwithNewTab(p.at(i)->text(3));
    }
}



#include "tsclirsview.moc"








