/***************************************************************************
 *   Copyright (C) 2003 by kzk(Kazuki Ohta)                                *
 *   mover@hct.zaq.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "tscrssview.h"
#include "tscrssview_dlg_configure.h"

#include <qapplication.h>
#include <qfile.h>
#include <qdom.h>
#include <qpoint.h>
#include <qptrlist.h>
#include <qtimer.h>
#include <qregexp.h>
#include <qbuffer.h>
#include <qpushbutton.h>
#include <qvbox.h>
#include <qstringlist.h>
#include <qlayout.h>
#include <qheader.h>
#include <qlabel.h>
#include <qframe.h>
#include <qcursor.h>
#include <qtimer.h>

#include <kglobal.h>
#include <kiconloader.h>
#include <kconfig.h>
#include <kdebug.h>
#include <klocale.h>
#include <kurl.h>
#include <klistview.h>
#include <kio/scheduler.h>
#include <kio/job.h>
#include <kio/jobclasses.h>

#define COLUMN_NAME 0
#define COLUMN_URL  1

TSCRSSView::TSCRSSView(const QString id,QWidget *parent, const char *name, WFlags f)
 : TaviaSidebarContents(id,parent, name, f)
{
    isClose = false;
    uid = id;
    tiplabel = NULL;
    config = new KConfig("taviasidebarrssviewrc",false,true);

    QVBoxLayout *v = new QVBoxLayout(this);

    m_listView = new KListView( this );
    m_listView->addColumn("Site Name");
    m_listView->addColumn("URL");
    m_listView->setSelectionMode( QListView::Single );
    m_listView->setSorting( COLUMN_URL, false );
    m_listView->header()->hide();
    m_listView->setRootIsDecorated(true);
    QObject::connect( m_listView, SIGNAL(doubleClicked(QListViewItem *, const QPoint&, int)),
		      this, SLOT(slotItemDoubleClicked(QListViewItem *, const QPoint&, int)) );
    QObject::connect( m_listView, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint&, int)),
		      this, SLOT(slotItemRightClicked(QListViewItem *, const QPoint&, int)) );
    QObject::connect( m_listView, SIGNAL(onItem(QListViewItem *)),
		      this, SLOT(slotOnItem(QListViewItem *)));
    QObject::connect( m_listView, SIGNAL(onViewport()),
		      this, SLOT(slotOnViewport()));

    refreshButton = new QPushButton( i18n("Refresh Contents"), this );
    QObject::connect( refreshButton, SIGNAL(clicked()),
		      this, SLOT(slotModifyAll()) );

    rssList.setAutoDelete(true);
    
    v->addWidget( m_listView );
    v->addWidget( refreshButton );

    loadConfig();
}


TSCRSSView::~TSCRSSView()
{
    if (tiplabel)
	delete tiplabel;

    rssList.clear();

    delete config;
}

/*!
    \fn TSCRSSView::createContents(const QString id)
 */
TaviaSidebarContents * TSCRSSView::createContents(const QString id)
{
    return(new TSCRSSView(id));
}

/*!
    \fn TSCRSSView::getContentsInfo()
 */
TaviaSidebarContentsInfo * TSCRSSView::getContentsInfo()
{
    TaviaSidebarContentsInfo *t = new TaviaSidebarContentsInfo();
    t->multiple = false;
    t->name = "RSSView";
    t->icon = "html";
    t->createContents = TSCRSSView::createContents;
    return(t);
}

/*!
    \fn TSCRSSView::getContentsTitle()
 */
const QString TSCRSSView::getContentsTitle()
{
    return( "RSSView" );
}

/*!
    \fn TSCRSSView::getContentsIcon()
 */
const QString TSCRSSView::getContentsIcon()
{
    return("html");
}

/*!
    \fn TSCRSSView::loadConfig()
 */
void TSCRSSView::loadConfig()
{
    /*
    if( !config->hasGroup( uid ) )
	writeDefault();
    */

    // clear
    rssList.clear();
    m_listView->clear();

    // read list
    config->setGroup( "TRSSViewBar" );
    QStringList nameList = config->readListEntry( "RSSNames" );
    QStringList::ConstIterator it = nameList.begin();
    QString item;
    while( it != nameList.end() ){
        item = *it;	
	if( !item.isEmpty() && config->hasGroup(item) ){
	    config->setGroup( item );
	    QString name = item;
	    KURL url( config->readPathEntry( "SourceFile" ) );
	    unsigned int interval = config->readEntry( "Interval" ).toInt();

	    insertRSS( url, interval, name );
	}
	++it;

	kdDebug() << "ITEM =" << item << endl;
    }
}

/*!
    \fn TSCRSSView::saveConfig()
 */
void TSCRSSView::saveConfig()
{
    config->setGroup( "TRSSViewBar" );
}

/*!
    \fn TSCRSSView::closeContents()
 */
void TSCRSSView::closeContents()
{
    config->deleteGroup(getUniqueId());
    isClose = true;
}

/*!
    \fn TSCRSSView::configure()
 */
void TSCRSSView::configure()
{
    TRSSConfigureDialog* d = new TRSSConfigureDialog( config );
    d->exec();

    if( d->isModify )
    loadConfig();

    delete d;
}


RSSObject::RSSObject( KURL u, unsigned int i, const QString& n )
{
    name = n;
    url = u;
    interval = i;

    modifyTimer = NULL;
    rootItem = NULL;

    if( interval )
	setModifyInterval( interval );
}

RSSObject::~RSSObject()
{
    if( modifyTimer ){
	delete modifyTimer;
    }
}

/*!
    \fn RSSObject::setModifyInterval( unsigned int intervalmin )
 */
void RSSObject::setModifyInterval( unsigned int intervalmin )
{
    if( modifyTimer == NULL ){
	modifyTimer = new QTimer( this, "trssviewer_intervaltimer" );

	kdDebug() << "TRSSViewPart : create timer for " << url.url() << endl;
    }
    
    unsigned int msec = intervalmin * 60000;
    modifyTimer->start( msec, FALSE );

    QObject::connect( modifyTimer, SIGNAL(timeout()),
		      this, SLOT(slotModifyRequested()) );
}

/*!
    \fn RSSObject::slotModifyRequested()
 */
void RSSObject::slotModifyRequested()
{
    kdDebug() << "RSSObject : modifyRequest = " << url.url() << endl;

    emit modifyRequest( this );
}

/*!
    \fn TRSSListViewItem::TRSSListViewItem::TRSSListViewItem( KListView* parent, QString n, KURL& u)
 */
TRSSListViewItem::TRSSListViewItem( KListView* parent, QString n, KURL& u )
    : KListViewItem( parent, n )
{
    name = n;
    url = u;
}

/*!
    \fn TRSSListViewItem::TRSSListViewItem( KListViewItem* parent, KListViewItem* after, QString n, KURL& u )
 */
TRSSListViewItem::TRSSListViewItem( KListViewItem* parent, KListViewItem* after, QString n, KURL& u )
    : KListViewItem( parent, after, n, u.url() )
{
    name = n;
    url = u;
}

/*!
    \fn TRSSListViewItem::deleteChildItems()
 */
void TRSSListViewItem::deleteChildItems()
{
    int totalChild = childCount();
    
    for( int i = 0; i < totalChild; i++ ){
	TRSSListViewItem* item = (TRSSListViewItem*)firstChild();

	if( item->childCount() )
	    item->deleteChildItems();

	delete (TRSSListViewItem*)item;
    }
}

/*!
    \fn TSCRSSView::insertRSS( KURL& url, unsigned int intervalmin, QString& name )
 */
bool TSCRSSView::insertRSS( KURL& url, unsigned int intervalmin, const QString& name )
{
    RSSObject* rssobj = NULL;
    
    // if already exist, only change intervalmin
    if( isInHistory( url ) )
	return true;
    
    // set RSSObject
    rssobj = new RSSObject( url, intervalmin, name );
    if( intervalmin ){
	QObject::connect( rssobj, SIGNAL(modifyRequest( RSSObject* )),
			  this, SLOT(slotModifyRSS( RSSObject* )) );
    }
    
    rssList.append( rssobj );
    
    // open!
    return openRSS( rssobj );
}

/*!
    \fn TSCRSSView::removeRSS( KURL& url )
 */
void TSCRSSView::removeRSS( KURL& url )
{
    RSSObject* rssobj = NULL;
    
    for ( rssobj = rssList.first(); rssobj; rssobj = rssList.next() ){
	if( QString::compare( rssobj->url.url(), url.url() ) == 0 ){
	    
	    kdDebug() << "TSCRSSView : remove view url = " << rssobj->url.url() << endl;
	    
	    delete rssobj->rootItem; /* remove items from list */
	    rssList.remove( rssobj );
	    break;
	}
    }
}

/*!
    \fn TSCRSSView::changeInterval( KURL& url, unsigned int intervalmin )
 */
bool TSCRSSView::changeInterval( KURL& url, unsigned int intervalmin )
{
    RSSObject* rssobj = findHistory( url );
    if( rssobj == NULL )
	return false;
    
    rssobj->setModifyInterval( intervalmin );
    return true;
}


/*!
    \fn TSCRSSView::isInHistory( KURL& url )
 */
bool TSCRSSView::isInHistory( const KURL& url )
{
    RSSObject* rssobj = NULL;
    
    for ( rssobj = rssList.first(); rssobj; rssobj = rssList.next() ){
	if( QString::compare( rssobj->url.url(), url.url() ) == 0 ){
	    return true;
	}
    }
    
    return false;
}

/*!
    \fn TSCRSSView::findHistory( KURL& url )
 */
RSSObject* TSCRSSView::findHistory( const KURL& url )
{
    RSSObject* rssobj = NULL;
    
    for ( rssobj = rssList.first(); rssobj; rssobj = rssList.next() ){
	if( QString::compare( rssobj->url.url(), url.url() ) == 0 ){
	    return rssobj;
	}
    }
    
    return NULL;
}

/*!
    \fn TSCRSSView::openRSS( RSSObject* rssobj )
 */
bool TSCRSSView::openRSS( RSSObject* rssobj )
{
    KURL url = rssobj->url;
    kdDebug() << "TSCRSSView: opening " << url.url() << endl;
    
    /* QFile cannnot deal with the file on NETWORK */
    if( url.isLocalFile() )
	return openLocalRSS( rssobj );
    else
	return openRemoteRSS( rssobj );
}

/*!
    \fn TSCRSSView::openLocalRSS( RSSObject* rssobj )
 */
bool TSCRSSView::openLocalRSS( RSSObject* rssobj )
{
    QFile file;
    file.setName( rssobj->url.path() );
    QDomDocument doc;
    doc.setContent( &file ); // Parse the file as a DOM document
    
    parseRSS( rssobj, doc );
    
    file.close();

    m_listView->sort();
    
    return true;
}


/*!
    \fn TSCRSSView::openRemoteRSS( RSSObject* rssobj )

    openRemoteRSS() -> slotData() -> slotJobFinished() -> parseRSS()
 */
bool TSCRSSView::openRemoteRSS( RSSObject* rssobj )
{
    rssobj->downloadData = new QBuffer;
    rssobj->downloadData->open(IO_WriteOnly);
    
    KIO::TransferJob* job = KIO::get( rssobj->url, false, false );

    QObject::connect( job, SIGNAL(data(KIO::Job *, const QByteArray &)),
		      this, SLOT(slotData(KIO::Job *, const QByteArray &)) );
    QObject::connect( job, SIGNAL(result(KIO::Job *)),
		      this, SLOT(slotJobFinished(KIO::Job *)) );
    
    return true;
}

/*!
    \fn TSCRSSView::parseRSS( RSSObject* rssobj, QDomDocument& doc )
 */
void TSCRSSView::parseRSS( RSSObject* rssobj, QDomDocument& doc )
{
    QDomElement elem = doc.documentElement(); // Get the root element(channel)
    if ( !elem.isNull() ){
	
	if( rssobj->rootItem == NULL ){
	    /* create root item of this rss */
	    QDomNode channelNode = elem.namedItem( "channel" );
	    QString rootTitle =  channelNode.namedItem( "title" ).toElement().text();
	    KURL url( channelNode.namedItem( "link" ).toElement().text() );

	    if( !rssobj->name.isEmpty() )
		rootTitle = rssobj->name;
	    
	    rssobj->rootItem = new TRSSListViewItem( m_listView, rootTitle, url );
	    setRootIcon( rssobj->rootItem );
			
	    kdDebug() << "TSCRSSView : new root item was created!" << endl;
	}
	    
	/* check <item> recursive */
	fillTreeView( elem, ((KListViewItem*)rssobj->rootItem) );
    }
}

/*!
    \fn TSCRSSView::fillTreeView( QDomElement& parentElem, KListViewItem* parent )

    check <title> and <list> in <item>
 */
void TSCRSSView::fillTreeView( QDomElement& parentElem, KListViewItem* parent )
{
    QDomElement docElem = parentElem;
    QDomNodeList links = docElem.elementsByTagName( "link" );
    QDomNodeList titles = docElem.elementsByTagName( "title" );
    KListViewItem *lastItem = 0L;
    
    QRegExp rx("^http://", false );
    for (uint i=1; i < titles.count(); ++i ){

	// skip <channel> node
	if( titles.item( i ).parentNode().toElement().tagName() != "item" )
	    continue;

	QString title = titles.item( i ).toElement().text();
	QString link = links.item( i ).toElement().text();
	if( (rx.search(title) == -1) && (rx.search(link) != -1) ){
	    kdDebug() << "TSCRSSView : title = " << title << endl;
	    kdDebug() << "TSCRSSView : link = " << link << endl;

	    KURL url( link );
	    lastItem = new TRSSListViewItem( parent, lastItem, title, url );
	}
    }
}

/*!
    \fn TSCRSSView::setRootIcon( QListViewItem* )
 */
void TSCRSSView::setRootIcon( QListViewItem* i )
{
    KIconLoader *loader = KGlobal::iconLoader();
    
    i->setPixmap( 0, loader->loadIcon( "bookmark", KIcon::Small, KIcon::SizeSmall ) );
}

/*!
    \fn TSCRSSView::writeDefault()
 */
void TSCRSSView::writeDefault()
{
    //$B$^$@$G$9!#(B
}

/*!
    \fn TSCRSSView::slotModifyRSS( RSSObject* rssobj )
 */
void TSCRSSView::slotModifyRSS( RSSObject* rssobj )
{
    // delete
    if( rssobj->rootItem )
    rssobj->rootItem->deleteChildItems();

    // reread
    openRSS( rssobj );
}

/*!
    \fn TSCRSSView::slotModifyAll()
 */
void TSCRSSView::slotModifyAll()
{
    RSSObject* rssobj;

    for ( rssobj = rssList.first(); rssobj; rssobj = rssList.next() ){
	slotModifyRSS( rssobj );
    }
}

/*!
    \fn TSCRSSView::slotItemDoubleClicked( QListViewItem* item, const QPoint& p, int c)
 */
void TSCRSSView::slotItemDoubleClicked( QListViewItem* item, const QPoint& p, int c )
{
    kdDebug() << "TSCRSSView : double clicked url = " << ((TRSSListViewItem*)item)->url.url() << endl;

    openURL( ((TRSSListViewItem*)item)->url );
}

/*!
    \fn TSCRSSView::slotItemRightClicked( QListViewItem* item, const QPoint& p, int c)
 */
void TSCRSSView::slotItemRightClicked( QListViewItem* item, const QPoint& p, int c )
{
}

/*!
    \fn TSCRSSView::slotData( KIO::Job* job, const QByteArray& data )
 */
void TSCRSSView::slotData( KIO::Job* job, const QByteArray& data )
{
    if( !data.isEmpty() ){
	RSSObject* rssobj = findHistory( ((KIO::SimpleJob*)job)->url() );
	if( !rssobj )
	    return;

	// store data
	rssobj->downloadData->writeBlock(data.data(), data.size());
    }
}

/*!
    \fn TSCRSSView::slotJobFinished( KIO::Job* job )
 */
void TSCRSSView::slotJobFinished( KIO::Job* job )
{
    if( job->error() ){
	job->showErrorDialog();
	return;
    }

    RSSObject* rssobj = findHistory( ((KIO::SimpleJob*)job)->url() );
    if( !rssobj )
	return;

    QDomDocument doc;
    doc.setContent( rssobj->downloadData->buffer() );
    
    parseRSS( rssobj, doc );

    delete rssobj->downloadData;
    rssobj->downloadData = 0;

    // $B$3$l0J30$K99?7=g$K%=!<%H$9$kJ}K!$O$J$$$b$N$+(B
    // URL$B$N9`L\$,>C$;$k$N$@$,!#(B
    m_listView->sort();
}

void TSCRSSView::slotMouseMoveEvent( QMouseEvent * e )
{
}

void TSCRSSView::slotOnItem( QListViewItem *item )
{
    if (tiplabel) {
	delete tiplabel;
	tiplabel = 0;
    }

    // rootitem
    if (item->childCount())
	return;

    QString labelText = item->text(0).insert(0, "Name : ") + " \n" + item->text(1).insert(0, "Url  : ") + " ";
    int screen = QApplication::desktop()->screenNumber(m_listView);
    tiplabel = new QLabel(QApplication::desktop()->screen(screen), "TOOLTIP", WStyle_StaysOnTop | WStyle_Customize | WStyle_NoBorder | WStyle_Tool | WX11BypassWM);
    KIconLoader *loader = KGlobal::iconLoader();
    tiplabel->setPixmap(loader->loadIcon("bookmark", KIcon::Small, KIcon::SizeSmall));
    tiplabel->setText(labelText);
    tiplabel->setAlignment( AlignAuto | AlignTop );
    tiplabel->setFrameStyle(QFrame::Plain | QFrame::Box);
    tiplabel->adjustSize();
    tiplabel->move(QPoint(QCursor::pos().x()+10, QCursor::pos().y()+15));
    tiplabel->show();
    tiplabel->raise();
    QTimer::singleShot(100,this,SLOT(slotUpdateTimer()));
}

void TSCRSSView::slotOnViewport()
{
    if (tiplabel) {
	delete tiplabel;
	tiplabel = 0;
    }
}

void TSCRSSView::slotUpdateTimer() {
    if (tiplabel == 0) return;
    if ((!hasMouse()) && (!tiplabel->hasMouse())) {
        slotOnViewport();
        return;
    } else {
        QTimer::singleShot(100,this,SLOT(slotUpdateTimer()));
    }
}


#include "tscrssview.moc"
