/***************************************************************************
 *   Copyright (C) 2003 by kzk(Kazuki Ohta)                                *
 *   mover@hct.zaq.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TSCRSSVIEW_H
#define TSCRSSVIEW_H

#include <taviasidebarcontents.h>

#include <qtimer.h>
#include <qbuffer.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qevent.h>

#include <klistview.h>

class QPushButton;

class KConfig;
class KURL;

class TSCRSSView;
class RSSObject;
class TRSSListViewItem;

/**
@author kzk(Kazuki Ohta)
*/
class TaviaSidebarContents;

class TSCRSSView : public TaviaSidebarContents
{
Q_OBJECT
public:
    TSCRSSView(const QString id,QWidget *parent = 0, const char *name = 0, WFlags f = 0);
    ~TSCRSSView();

    static TaviaSidebarContentsInfo * getContentsInfo();
    static TaviaSidebarContents * createContents(const QString id);

    virtual const QString getContentsTitle();
    virtual const QString getContentsIcon();

    virtual void closeContents();

    bool insertRSS( KURL& url, unsigned int intervalmin = 0, const QString& name = QString::null );
    void removeRSS( KURL& url );
    bool changeInterval( KURL& url, unsigned int intervalmin );

protected:
    void writeDefault();
    void loadConfig();
    void saveConfig();

    bool isInHistory( const KURL& url );
    RSSObject* findHistory( const KURL& url );
    
    bool openRSS( RSSObject* rssobj );
    bool openLocalRSS( RSSObject* rssobj );
    bool openRemoteRSS( RSSObject* rssobj );
    
    void parseRSS( RSSObject* rssobj, QDomDocument& doc );
    void fillTreeView( QDomElement& parentElem, KListViewItem* parent );

    void setRootIcon( QListViewItem* );

signals:
    void selected( KURL& );

public slots:
    virtual void configure();

    void slotModifyRSS( RSSObject* rssobj );
    void slotModifyAll();
    
    void slotItemDoubleClicked( QListViewItem *item, const QPoint& p, int c);
    void slotItemRightClicked( QListViewItem *item, const QPoint& p, int c);
    
    void slotData( KIO::Job *, const QByteArray & );
    void slotJobFinished( KIO::Job * );
    void slotMouseMoveEvent( QMouseEvent * );
    void slotOnItem( QListViewItem * );
    void slotOnViewport();
    void slotUpdateTimer();

protected:
    KConfig *config;

    KListView* m_listView;
    QPushButton* refreshButton;
    QLabel* tiplabel;
    
    QPtrList<RSSObject> rssList;

    QString uid;
    bool isClose;
};

class RSSObject : public QObject
{
    Q_OBJECT
public:
    RSSObject( KURL url, unsigned int interval = 0, const QString& name = QString::null );
    ~RSSObject();

    void setModifyInterval( unsigned int intervalmin  );
signals:
    void modifyRequest( RSSObject* rssobj );
public slots:
    void slotModifyRequested();

public:
    QString name;
    KURL url; // url of rssfile
    unsigned int interval;
    
    TRSSListViewItem* rootItem; // root list item of this rss
    QBuffer* downloadData;

private:
    QTimer* modifyTimer;
};

class TRSSListViewItem : public KListViewItem
{
public:
    TRSSListViewItem( KListView* parent, QString n, KURL& u );
    TRSSListViewItem( KListViewItem* parent, KListViewItem* after, QString n, KURL& u );

    void deleteChildItems();

    QString name;
    KURL url;
};

#endif
