/***************************************************************************
 *   Copyright (C) 2003 by kzk(Kazuki Ohta)                                *
 *   mover@hct.zaq.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "tscrssview_dlg_configure.h"

#include <qstring.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qstringlist.h>

#include <kconfig.h>
#include <kurl.h>
#include <klocale.h>
#include <klistview.h>
#include <klineedit.h>
#include <knuminput.h>
#include <kmessagebox.h>
#include <kdebug.h>

#define COLUMN_NAME     0
#define COLUMN_URL      1
#define COLUMN_INTERVAL 2

TRSSConfigureDialog::TRSSConfigureDialog( KConfig* conf )
    :  KDialogBase( KDialogBase::Plain, i18n("TRSS Configure Dialog"),
		    Ok|Cancel, Ok )
{
    config = conf;
    isModify = false;

    // set GUI
    QFrame* frame = plainPage();
    QVBoxLayout* frameLayout = new QVBoxLayout( frame );
    QVBox* mainQVBox = new QVBox( frame );

    siteListView = new KListView( mainQVBox );
    siteListView->setSelectionMode( QListView::Single );
    siteListView->addColumn( i18n("Name") );
    siteListView->addColumn( i18n("RSS Url") );
    siteListView->addColumn( i18n("Interval(minute)") );

    QHBox* downQHBox = new QHBox( mainQVBox );
    addButton = new QPushButton( i18n("Add"), downQHBox );
    modifyButton = new QPushButton( i18n("Modify"), downQHBox );
    deleteButton = new QPushButton( i18n("Delete"), downQHBox );

    frameLayout->addWidget( mainQVBox );

    // set S&S
    QObject::connect( this, SIGNAL(okClicked()),
		      this, SLOT(slotApply()) );

    QObject::connect( addButton, SIGNAL(clicked()),
		      this, SLOT(slotAdd()) );
    QObject::connect( modifyButton, SIGNAL(clicked()),
		      this, SLOT(slotModify()) );
    QObject::connect( deleteButton, SIGNAL(clicked()),
		      this, SLOT(slotDeleteItem()) );

    readEntry();
}

TRSSConfigureDialog::~TRSSConfigureDialog(){
}

void TRSSConfigureDialog::readEntry()
{
    // read list
    config->setGroup( "TRSSViewBar" );
    QStringList nameList = config->readListEntry( "RSSNames" );
    QStringList::ConstIterator it = nameList.begin();
    QString item;
    while( it != nameList.end() ){
	item = *it;

	if( !item.isEmpty() && config->hasGroup(item) ){
	    config->setGroup( item );
	    QString name = item;
	    QString url = config->readPathEntry( "SourceFile" );
	    QString interval = config->readEntry( "Interval" );

	    KListViewItem* item = new KListViewItem( siteListView, name, url, interval );
	}

	++it;
    }

    kdDebug() << "TRSSConfigureDialog : readEntry()" << endl;
}

void TRSSConfigureDialog::writeEntry()
{
    QStringList nameList;

    for( int i = 0; i < siteListView->childCount(); i++ ){
	QListViewItem* item = siteListView->itemAtIndex( i );
    updateNameList();

	KURL url( item->text( COLUMN_URL ) );
	RSSObject* rssobj = new RSSObject( url, (unsigned int)item->text( COLUMN_INTERVAL ).toInt(), item->text( COLUMN_NAME ) );

	nameList << rssobj->name;

	// write item
	config->setGroup( rssobj->name );
	config->writePathEntry( "SourceFile", rssobj->url.url() );
	config->writeEntry( "Interval", rssobj->interval );
    }
    config->sync();

    kdDebug() << "TRSSConfigureDialog : writeEntry()" << endl;
}

void TRSSConfigureDialog::updateNameList()
{
    QStringList nameList;

    for( int i = 0; i < siteListView->childCount(); i++ ){
	QListViewItem* item = siteListView->itemAtIndex( i );

	nameList << item->text( COLUMN_NAME );
    }

    // write namelist
    config->setGroup( "TRSSViewBar" );
    config->writeEntry( "RSSNames", nameList );

    config->sync();
    
    kdDebug() << "TRSSConfigureDialog : writeEntry()" << endl;
}

void TRSSConfigureDialog::slotApply()
{
    writeEntry();

    kdDebug() << "TRSSConfigureDialog : Apply" << endl;
}

void TRSSConfigureDialog::slotAdd()
{
    // open dialog
    TRSSAddDialog* d = new TRSSAddDialog();
    int code = d->exec();
    if( code != TRSSAddDialog::Accepted ){
	delete d;
	return;
    }

    // add item to listview
    KListViewItem* item = new KListViewItem( siteListView, d->getSiteName(), d->getSiteURL().url(), QString::number( d->getInterval() ) );

    delete d;

    isModify = true;

    writeEntry();
}

void TRSSConfigureDialog::slotModify()
{
    // get RSS data
    QListViewItem* item = siteListView->selectedItem();
    if( item == NULL )
	return;

    // open dialog
    KURL url( item->text(COLUMN_URL) );
    TRSSAddDialog* d = new TRSSAddDialog( item->text(COLUMN_NAME), url, (unsigned int)item->text(COLUMN_INTERVAL).toInt() );
    int code = d->exec();
    if( code != TRSSAddDialog::Accepted )
	return;

    // modify item
    item->setText( COLUMN_NAME, d->getSiteName() );
    item->setText( COLUMN_URL, d->getSiteURL().url() );
    item->setText( COLUMN_INTERVAL, QString::number( d->getInterval() ) );

    delete d;

    isModify = true;
}

void TRSSConfigureDialog::slotDeleteItem()
{
    // remove item from list
    QListViewItem* item = siteListView->selectedItem();
    if( item != NULL ){
	// write item
	config->deleteGroup( item->text( COLUMN_NAME ) );

	delete (KListViewItem*)item;
    }
    writeEntry();

    isModify = true;
}


TRSSAddDialog::TRSSAddDialog()
    : KDialogBase( KDialogBase::Plain, "Add RSS Site", Ok|Cancel, Ok )
{
    setupGUI();
}

TRSSAddDialog::TRSSAddDialog( QString name, KURL& url, unsigned int interval ) // for Modify
    : KDialogBase( KDialogBase::Plain, "Modify RSS Site", Ok|Cancel, Ok )
{
    setupGUI();

    nameEdit->setText( name );
    rssUrlEdit->setText( url.url() );
    intervalSpinBox->setValue( interval );
}

TRSSAddDialog::~TRSSAddDialog(){
}

void TRSSAddDialog::setupGUI()
{
    QFrame* frame = plainPage();
    QVBoxLayout* frameLayout = new QVBoxLayout( frame );
    
    QVBox* mainQVBox = new QVBox( frame );

    QLabel* nameLabel = new QLabel( i18n("Name"), mainQVBox );
    nameEdit = new KLineEdit( mainQVBox );
    nameLabel->setBuddy( nameEdit );
    nameEdit->setFocus();

    QLabel* rssUrlLabel = new QLabel( i18n("RSS Url") , mainQVBox );
    rssUrlEdit = new KLineEdit( mainQVBox );
    rssUrlLabel->setBuddy( rssUrlEdit );
    rssUrlEdit->setText( "http://" );

    QLabel* intervalLabel = new QLabel( i18n("Interval(minute)") , mainQVBox );
    intervalSpinBox = new KIntSpinBox( mainQVBox );
    intervalLabel->setBuddy( intervalSpinBox );
    intervalSpinBox->setValue( 0 );

    frameLayout->addWidget( mainQVBox );

    QObject::connect( this, SIGNAL(okClicked()),
		      this, SLOT(slotApply()) );
}

QString TRSSAddDialog::getSiteName()
{
    return nameEdit->text();
}

KURL TRSSAddDialog::getSiteURL()
{
    KURL url( rssUrlEdit->text() );
    return url;
}

unsigned int TRSSAddDialog::getInterval()
{
    return (unsigned int)intervalSpinBox->value();
}

void TRSSAddDialog::slotApply()
{
    /**
     * TODO
     * $B%@%$%"%m%0$rJD$8$:$K7Y9p$r=P$9$K$O$I$&$9$l$P$h$$$+!)(B
     */
    /*
    if( getSiteName().isEmpty() ){
	KMessageBox::sorry( this, "[Name] isn't filled!" );
    }
    */
}
