/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "tsessbbox.h"

TSESSBBox::TSESSBBox(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
    freeScale = false;
    buttons.setAutoDelete(true);
    //vertical = true;
    lay = 0;
    hlay = 0;
    upButton = new QToolButton(this,"upButton");
    downButton = new QToolButton(this,"downButton");
    box = new TSESSScrollView(this,"box");
    box->setVScrollBarMode(QScrollView::AlwaysOff);
    box->setHScrollBarMode(QScrollView::AlwaysOff);
    box->enableClipper(false);
    connect(box,SIGNAL(extWheelActionUp()),this,SIGNAL(extWheelActionUp()));
    connect(box,SIGNAL(extWheelActionDown()),this,SIGNAL(extWheelActionDown()));
    connect(box,SIGNAL(tMouseMoveEvent(int,int)),this,SLOT(hoverCheck(int,int)));
    connect(upButton,SIGNAL(clicked()),this,SLOT(upScroll()));
    connect(downButton,SIGNAL(clicked()),this,SLOT(downScroll()));
    setButtonSize(120);
    layout();
    //setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
}





/*!
    \fn TSESSBBox::layout()
 */
void TSESSBBox::layout()
{
    if (height() > width()) {
        vertical = true;
        if (hlay != 0) {
            hlay->remove(upButton);
            hlay->remove(box);
            hlay->remove(downButton);
            delete hlay;
            hlay = 0;
        }
        upButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("2uparrow",KIcon::Toolbar,KIcon::SizeSmall));
        downButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("2downarrow",KIcon::Toolbar,KIcon::SizeSmall));
        if (lay  == 0) lay = new QVBoxLayout(this);
        lay->addWidget(upButton);
        lay->addWidget(box);
        lay->addWidget(downButton);
    } else {
        vertical = false;
        if (lay != 0) {
            lay->remove(upButton);
            lay->remove(box);
            lay->remove(downButton);
            delete lay;
            lay = 0;
        }
        upButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("2leftarrow",KIcon::Toolbar,KIcon::SizeSmall));
        downButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("2rightarrow",KIcon::Toolbar,KIcon::SizeSmall));
        if (hlay  == 0) hlay = new QHBoxLayout(this);
        hlay->addWidget(upButton);
        hlay->addWidget(box);
        hlay->addWidget(downButton);
    }
}


TSESSBBox::~TSESSBBox()
{
}

/*!
    \fn TSESSBBox::addTSESSButton
 */
void TSESSBBox::addTSESSButton()
{
    TSESSButton *b = new TSESSButton(box->viewport());
    b->show();
    b->setMiniButton(TSESSButton::Right);
    b->hideMiniButton();
    b->setButtonSize(buttonSize);

    box->addChild(b,0,buttonSize * buttons.count());
    connect(b,SIGNAL(buttonPressed(TSESSButton*)),this,SLOT(buttonPressed(TSESSButton*)));
    connect(b,SIGNAL(miniButtonClicked(TSESSButton::MiniButton,TSESSButton *)),this,SLOT(miniButtonClicked(TSESSButton::MiniButton,TSESSButton *)));
    connect(b,SIGNAL(dropUrlList(TSESSButton *,const KURL::List)),this,SLOT(slotDropUrlList(TSESSButton*,const KURL::List)));
    buttons.append(b);
    TSESSButtonRedraw();
}

/*!
    \fn TSESSBBox::hover(TSESSButton *b)
 */
void TSESSBBox::hover(TSESSButton *b)
{
    for(unsigned int i = 0;i < buttons.count();i ++) {
        if (b == buttons.at(i)) {
            if (buttons.at(i)->getButtonStyle() == TSESSButton::Plain) {
                emit buttons.at(i)->tButtonRaised();
                emit buttons.at(i)->showMiniButton();
            }
        } else {
            if (buttons.at(i)->getButtonStyle() == TSESSButton::Raised) {
                emit buttons.at(i)->tButtonPlain();
                emit buttons.at(i)->hideMiniButton();
            }
        }
    }
}

/*!
    \fn TSESSBBox::getButtonNum(TSESSButton *b)
 */
int TSESSBBox::getButtonNum(TSESSButton *b)
{
    for(unsigned int i = 0;i < buttons.count();i ++) {
        if (b == buttons.at(i)) return((int)i);
    }
    return(-1);
}

/*!
    \fn TSESSBBox::buttonPressed(TSESSButton *b)
 */
void TSESSBBox::buttonPressed(TSESSButton *b)
{
/*    if ((selected >= 0) && (selected < buttons.count())) {
        emit buttons.at((unsigned int)selected)->tButtonPlain();
        emit buttons.at((unsigned int)selected)->hideMiniButton();
    }
    selected = getButtonNum(b);
    emit buttons.at((unsigned int)selected)->tButtonSunken();
    emit buttons.at((unsigned int)selected)->showMiniButton();
*/
    emit buttonSelected(getButtonNum(b));
    emit select(getButtonNum(b));

}

/*!
    \fn TSESSBBox::select(int num)
 */
void TSESSBBox::select(int num)
{
    // ޡȤʤ
    //emit buttonPressed(buttons.at((unsigned int)num));
    TSESSButton *b = buttons.at((unsigned int)num);
    for(unsigned int i = 0;i < buttons.count();i ++) {
        if (b == buttons.at(i)) {
            emit buttons.at(i)->tButtonSunken();
            emit buttons.at(i)->showMiniButton();
            // 򤷤ܥ򸫤褦˥롣
            if (vertical) {
                box->setContentsPos(0,buttons.at(i)->height() * i);
            } else {
                box->setContentsPos(buttons.at(i)->width() * i,0);
            }
            //emit buttonSelected((int)i);
        } else {
            emit buttons.at(i)->tButtonPlain();
            emit buttons.at(i)->hideMiniButton();
        }
    }

}

/*!
    \fn TSESSBBox::getTSESSButton(int num)
 */
TSESSButton * TSESSBBox::getTSESSButton(int num)
{
    return(buttons.at((unsigned int)num));
}

/*!
    \fn TSESSBBox::delTSESSButton(int num)
 */
int TSESSBBox::delTSESSButton(int num)
{
    if (buttons.count() > (unsigned int)num) {
        //delete getTSESSButton(num);
        buttons.remove((unsigned int)num);
        TSESSButtonRedraw();
        return(buttons.count());
    }
    return(-1);
}

/*!
    \fn TSESSBBox::miniButtonClicked(TSESSButton::MiniButton b,TSESSButton * sb)
 */
void TSESSBBox::miniButtonClicked(TSESSButton::MiniButton b,TSESSButton * sb)
{
    if (b == TSESSButton::Left) emit leftMiniButtonClicked(getButtonNum(sb));
    else if (b == TSESSButton::Right) emit rightMiniButtonClicked(getButtonNum(sb));
}

/*!
    \fn TSESSBBox::hoverCheck(int x,int y)
 */
void TSESSBBox::hoverCheck(int x,int y)
{
    for(unsigned int i = 0;i < buttons.count();i ++) {
        if ((x >= box->childX(buttons.at(i))) &&
            (x <= (box->childX(buttons.at(i)) + buttons.at(i)->width())) &&
            (y >= box->childY(buttons.at(i))) &&
            (y <= (box->childY(buttons.at(i)) + buttons.at(i)->height()))) {
            emit hover(buttons.at(i));
            return;
        }
    }
    emit hover((TSESSButton*) 0);
}

/*!
    \fn TSESSBBox::TSESSButtonRedraw
 */
void TSESSBBox::TSESSButtonRedraw()
{
    int full = 0;
    for(unsigned int i = 0;i < buttons.count();i ++) {
        if (vertical) {
            box->moveChild(buttons.at(i),0,full);
            full += buttons.at(i)->height();
        } else {
            box->moveChild(buttons.at(i),full,0);
            full += buttons.at(i)->width();
        }
    }
    if (buttons.count() != 0) {
        if (vertical) {
            emit box->resizeContents(buttons.at(0)->width(),full);
        } else {
            emit box->resizeContents(full,buttons.at(0)->height());
        }
    }
    kdDebug() << "BUTTONS.REDRAW: " << buttons.count() << endl;
}

/*!
    \fn TSESSBBox::slotDropUrlList(TSESSButton *b,const KURL::List urllist)
 */
void TSESSBBox::slotDropUrlList(TSESSButton *b,const KURL::List urllist)
{
    emit dropUrlList(getButtonNum(b),urllist);
}

/*!
    \fn TSESSBBox::downScroll()
 */
void TSESSBBox::downScroll()
{
    if (vertical) box->scrollBy(0,120);
    else box->scrollBy(120,0);
}

/*!
    \fn TSESSBBox::upScroll()
 */
void TSESSBBox::upScroll()
{
    if (vertical) box->scrollBy(0,-120);
    else box->scrollBy(-120,0);
}

/*!
    \fn TSESSBBox::getButtonSize()
 */
int TSESSBBox::getButtonSize()
{
    return(buttonSize);
}

/*!
    \fn TSESSBBox::setButtonSize(int size)
 */
void TSESSBBox::setButtonSize(int size)
{
    buttonSize = size;
    if (!freeScale) {
        if (vertical) {
            //setFixedWidth(size + 2);
            setMinimumWidth(size + 2);
            setMaximumWidth(size + 2);
            setMaximumHeight(10000);
            setMinimumHeight(0);
            setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);


        } else {
            //setFixedHeight(size + 2);
            setMinimumHeight(size + 2);
            setMaximumHeight(size + 2);
            setMaximumWidth(10000);
            setMinimumWidth(0);
            setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);


        }
    }
    for (unsigned int i = 0;i < buttons.count();i ++) {
        buttons.at(i)->setButtonSize(size);
    }
    updateGeometry();
    TSESSButtonRedraw();
}

/*!
    \fn TSESSBBox::setVertical(bool v)
 */
void TSESSBBox::setVertical(bool v)
{
    vertical = v;
    layout();
    //setButtonSize(buttonSize);
    //setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
}

/*!
    \fn TSESSBBox::resizeEvent(QResizeEvent *e)
 */
void TSESSBBox::resizeEvent(QResizeEvent *e)
{
    if (!freeScale) {
        layout();
        setButtonSize(buttonSize);
        QWidget::resizeEvent(e);
        return;
    }
    emit resized();
    kdDebug() << "!!!resized!!!" << endl;
    if (height() > width()) {
        if (!vertical) layout();
        setButtonSize(width());
    } else {
        if (vertical) layout();
        setButtonSize(height());
    }
    QWidget::resizeEvent(e);
}


/*!
    \fn TSESSBBox::isFreeScale()
 */
bool TSESSBBox::isFreeScale()
{
    return(freeScale);
}


/*!
    \fn TSESSBBox::setFreeScale(bool b)
 */
void TSESSBBox::setFreeScale(bool b)
{
    if (b) {
        setMaximumHeight(10000);
        setMinimumHeight(0);
        setMaximumWidth(10000);
        setMinimumWidth(0);
        setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    } else {
        setButtonSize(buttonSize);
    }
    freeScale = b;
}
