/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "tsessbutton.h"
#include <qfont.h>

TSESSButton::TSESSButton(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
    // drop.
    setAcceptDrops(true);

    buttonSize = 80;

    setPaletteBackgroundColor(QColor(85,85,82));
    setPaletteForegroundColor(QColor(255,255,255));
    setMouseTracking(true);
    tFrame = new QFrame(this,"tFrame");
    tFrame->setMouseTracking(true);
    tButtonPlain();
    tLay = new QGridLayout(this,1,1,1,1,"tLay");
    tVBoxLay = new QVBoxLayout(tFrame,2,2,"tVBoxLay");
    tIcon = new QLabel(tFrame,"tIcon");
    tIcon->setMouseTracking(true);
    tIcon->setPixmap(tIconPixmap);
    tIcon->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Expanding);
    tVBoxLay->addWidget(tIcon);
    tHBoxLay = new QHBoxLayout(0,2,2,"tHBoxLay");
    tLeftButton = new TSESSMiniButton(tFrame,"tLeftButton");
    tHBoxLay->addWidget(tLeftButton);
    tLabel = new KSqueezedTextLabel(tFrame);
    tLabel->setText("");
    tLabel->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    tLabel->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    tHBoxLay->addWidget(tLabel);
    tRightButton = new TSESSMiniButton(tFrame,"tRightButton");
    tHBoxLay->addWidget(tRightButton);
    tVBoxLay->addLayout(tHBoxLay);
    tLay->addWidget(tFrame,0,0);
    tLay->activate();
    setIcon(QPixmap());
    //setButtonSize(80);

    connect(tLeftButton,SIGNAL(clicked()),this,SLOT(leftMiniButtonClicked()));
    connect(tRightButton,SIGNAL(clicked()),this,SLOT(rightMiniButtonClicked()));}


TSESSButton::~TSESSButton()
{
}

/*!
    \fn TSESSButton::setIcon(const QPixmap &pixmap)
 */
void TSESSButton::setIcon(const QPixmap &pixmap)
{
    tIconPixmap = pixmap;
    tOrigIconPixmap = pixmap;
    if (pixmap.isNull()) {
        tOrigIconPixmap.resize(100,82);
        tOrigIconPixmap.fill(QColor(255,255,255));
    }
    setIcon();
}

/*!
    \fn TSESSButton::setIcon()
 */
void TSESSButton::setIcon()
{
    if (tIconPixmap.width() != (buttonSize - 10)) {
        //tIconPixmap = tOrigIconPixmap;
        QPixmap pix;
        QImage img = tOrigIconPixmap.convertToImage();
        kdDebug() << "!!!buttonSize: " << buttonSize << endl;
        int s = ((buttonSize - 10) * 82) / 100;
        //pix.resize(buttonSize - 10,s);
        pix.convertFromImage(img.smoothScale(buttonSize - 10,s));
        tIconPixmap = pix;
        tIcon->setPixmap(tIconPixmap);
        //tIcon->setPixmap(pix);
    }

}

/*!
    \fn TSESSButton::setLabel(const QString &label)
 */
void TSESSButton::setLabel(const QString &label)
{
    tLabel->setText(label);
}

/*!
    \fn TSESSButton::tButtonRaised()
 */
void TSESSButton::tButtonRaised()
{
    buttonStyle = Raised;
    tFrame->setLineWidth(1);
    tFrame->setMidLineWidth(1);
    tFrame->setFrameShape(QFrame::WinPanel);
    tFrame->setFrameShadow(QFrame::Raised);
}

/*!
    \fn TSESSButton::tButtonSunken()
 */
void TSESSButton::tButtonSunken()
{
    buttonStyle = Sunken;
    tFrame->setLineWidth(1);
    tFrame->setMidLineWidth(1);
    tFrame->setFrameShape(QFrame::WinPanel);
    tFrame->setFrameShadow(QFrame::Sunken);
}

/*!
    \fn TSESSButton::tButtonPlain()
 */
void TSESSButton::tButtonPlain()
{
    buttonStyle = Plain;
    tFrame->setFrameShape(QFrame::NoFrame);
    tFrame->setFrameShadow(QFrame::Plain);
}

/*!
    \fn TSESSButton::mousePressEvent(QMouseEvent *e)
 */
void TSESSButton::mousePressEvent(QMouseEvent *e)
{
    emit buttonPressed(this);
}

/*!
    \fn TSESSButton::mouseReleaseEvent(QMouseEvent *e)
 */
void TSESSButton::mouseReleaseEvent(QMouseEvent *e)
{
//    tButtonSunken();
}

/*!
    \fn TSESSButton::showMiniButton()
 */
void TSESSButton::showMiniButton()
{
    if ((miniButtons == Left) || (miniButtons == Both))
        tLeftButton->show();
    if ((miniButtons == Right) || (miniButtons == Both))
        tRightButton->show();
}

/*!
    \fn TSESSButton::hideMiniButton()
 */
void TSESSButton::hideMiniButton()
{
    if ((miniButtons == Right) || (miniButtons == Both))
        tRightButton->hide();
    if ((miniButtons == Left) || (miniButtons == Both))
        tLeftButton->hide();
}


/*!
    \fn TSESSButton::getButtonStyle
 */
TSESSButton::Styles TSESSButton::getButtonStyle()
{
    return(buttonStyle);
}

/*!
    \fn TSESSButton::setMiniButton(MiniButton b)
 */
void TSESSButton::setMiniButton(MiniButton b)
{
    if ((b == Left) || (b == None)) tRightButton->hide();
    if ((b == Right) || (b == None)) tLeftButton->hide();
    miniButtons = b;
}

/*!
    \fn TSESSButton::leftMiniButtonClicked()
 */
void TSESSButton::leftMiniButtonClicked()
{
    emit miniButtonClicked(Left,this);
}

/*!
    \fn TSESSButton::rightMiniButtonClicked()
 */
void TSESSButton::rightMiniButtonClicked()
{
    emit miniButtonClicked(Right,this);
}

/*!
    \fn TSESSButton::dragEnterEvent(QDragEnterEvent *event)
 */
void TSESSButton::dragEnterEvent(QDragEnterEvent *event)
{
    event->accept(KURLDrag::canDecode(event));
}

/*!
    \fn TSESSButton::dropEvent(QDropEvent *event)
 */
void TSESSButton::dropEvent(QDropEvent *event)
{
    KURL::List urllist;
    if (KURLDrag::decode(event,urllist)){
        kdDebug() << "Drop URL" << endl;
        emit dropUrlList(this,urllist);
    }
}

/*!
    \fn TSESSButton::setButtonSize(int size)
 */
unsigned int TSESSButton::setButtonSize(int size)
{
    setMaximumWidth(size);
    setMinimumWidth(size);
    setMaximumHeight(size);
    setMinimumHeight(size);
    tFrame->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    resize(QSize(size,size));
    buttonSize = size;
    setIcon();
    QFont f = font();
    f.setPixelSize((size - (((buttonSize - 10) * 82) / 100)) / 2);
    tLabel->setFont(f);
    return(size);
}






















