/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "tsessview.h"

TSESSView::TSESSView(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
    selected = -1;
    vertical = true;
    frames.setAutoDelete(true);
    hlay = new QHBoxLayout(this);
    lay = hlay;
    vlay = 0;
    tSessBBox = new TSESSBBox(this,"tSessBBox");
    lay->addWidget(tSessBBox);
//    tSessFrame = new QFrame(this,"tSessFrame");
    dummyFrame = new QFrame(this,"dummyFrame");
    dummyFrame->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    lay->addWidget(dummyFrame);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    connect(tSessBBox,SIGNAL(buttonSelected(int)),this,SLOT(setSelected(int)));
    connect(tSessBBox,SIGNAL(dropUrlList(int,const KURL::List)),this,SLOT(slotDropUrlList(int,const KURL::List)));

}


TSESSView::~TSESSView()
{
}

/*!
    \fn TSESSView::tAddFrame()
 */
int TSESSView::tAddFrame()
{
    dummyFrame->hide();
    tSessBBox->addTSESSButton();
    QFrame *f = new QFrame(this,"tSessFrames");
    kdDebug() << "tAddFrame Frame added: " << f << endl;
    f->show();
    lay->addWidget(f);
    //f->hide();
    f->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    frames.append(f);
/*    if (selected == -1) {
        selected = 0;
        setSelected(selected);
    }
*/
    //selected = frames.count() - 1;
    //tSessBBox->select(selected);
    //setSelected(selected);
    return(frames.count() - 1);
}

/*!
    \fn TSESSView::tDelFrame(int num)
 */
int TSESSView::tDelFrame(int num)
{
    if (frames.count() > (unsigned int)num) {
        //delete getFrame(num);
        frames.remove((unsigned int)num);
        if (frames.count() == 0) {
            dummyFrame->show();
            selected = -1;
        }
        return(frames.count());
    }
    return(-1);
}

/*!
    \fn TSESSView::delFrame(int num)
 */
void TSESSView::delFrame(int num)
{
    // ˥ʥס
    emit TSESSDeleted(num);
    tDelFrame(num);
    tSessBBox->delTSESSButton(num);
    if ((unsigned int)selected >= frames.count()) selected --;
    //tSessBBox->TSESSButtonRedraw();
    setSelected(selected);
}

/*!
    \fn TSESSView::addFrame()
 */
int TSESSView::addFrame()
{
    return(tAddFrame());
}

/*!
    \fn TSESSView::tSelect(int num)
 */
void TSESSView::tSelect(int num)
{
    selected = num;
    //emit tSessBBox->select(num);

    for(unsigned int i = 0;i < frames.count();i ++) {
        if (i == (unsigned int)num) {
            frames.at(i)->show();
            kdDebug() << "!!!found frame" << endl;
        } else {
            frames.at(i)->hide();
        }
    }

}

/*!
    \fn TSESSView::getSelected()
 */
int TSESSView::getSelected()
{
    return(selected);
}

/*!
    \fn TSESSView::*getFrame(int num)
 */
QFrame * TSESSView::getFrame(int num)
{
    return(frames.at((unsigned int)num));
}

/*!
    \fn TSESSView::setSelected(int num)
 */
void TSESSView::setSelected(int num)
{
    tSelect(num);
    emit tSessBBox->select(num);
    emit changed(num);
}

/*!
    \fn TSESSView::getTSESSButton(int num)
 */
TSESSButton * TSESSView::getTSESSButton(int num)
{
    return(tSessBBox->getTSESSButton(num));
}

/*!
    \fn TSESSView::getNumFromFrame(QFrame *f)
 */
int TSESSView::getNumFromFrame(QFrame *f)
{
    for(unsigned int i = 0;i < frames.count();i ++) {
        if (frames.at(i) == f) {
            return((int)i);
        }
    }
    return(-1);
}

/*!
    \fn TSESSView::slotDropUrlList(int num,const KURL::List urllist)
 */
void TSESSView::slotDropUrlList(int num,const KURL::List urllist)
{
    emit dropUrlList(num,urllist);
}

/*!
    \fn TSESSView::count()
 */
int TSESSView::count()
{
    return(frames.count());
}

/*!
    \fn TSESSView::setBBoxShow(bool b)
 */
void TSESSView::setBBoxShow(bool b)
{
    if (b) tSessBBox->show();
    else tSessBBox->hide();
    emit TSESSBBoxShown(b);
}

/*!
    \fn TSESSView::setButtonSize(int size)
 */
void TSESSView::setButtonSize(int size)
{
    tSessBBox->setButtonSize(size);
}



/*!
    \fn TSESSView::setVerticalBox(bool v)
 */
void TSESSView::setVerticalBox(bool v)
{
    if (v == vertical) return;
    vertical = v;
    tSessBBox->setVertical(v);
    lay->remove(tSessBBox);
    lay->remove(dummyFrame);
    for(unsigned int i = 0;i < frames.count();i ++) {
        lay->remove(frames.at(i));
    }
    if (v) {
        if (vlay != 0) {
            delete vlay;
            vlay = 0;
        }
        hlay = new QHBoxLayout(this);
        lay = hlay;
    } else {
        if (hlay != 0) {
            delete hlay;
            hlay = 0;
        }
        vlay = new QVBoxLayout(this);
        lay = vlay;
    }

    lay->addWidget(tSessBBox);
    lay->addWidget(dummyFrame);
    for(unsigned int i = 0;i < frames.count();i ++) {
        lay->addWidget(frames.at(i));
    }
    updateGeometry();
}

/*!
    \fn TSESSView::getBBox()
 */
TSESSBBox * TSESSView::getBBox()
{
    return(tSessBBox);
}
