/*
 * Copyright (C) 2003 TAM(Teppei Tamra) <tam-t@par.odn.ne.jp>
 */

#include "tavia.h"
#include "taviatab.h"
#include "taviahtmlview.h"


#include <qlayout.h>
#include <qframe.h>
#include <qlabel.h>
#include <qcursor.h>
#include <qsize.h>
#include <kdockwidget.h>
#include <kaction.h>
#include <kmainwindow.h>
#include <khelpmenu.h>
#include <kstatusbar.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <klocale.h>

tavia::tavia()
    : KDockMainWindow( 0, "tavia" )
{
    resizeReady = true;

    // set the shell's ui resource file
    setXMLFile("taviaui.rc");
    //setStandardToolBarMenuEnabled(true);
    KStdAction::quit(this,SLOT(close()),actionCollection());
    new KAction(i18n("&Dock"),"view_choose",SHIFT+CTRL+Key_T,this,SLOT(slotDockMenuPopup()),actionCollection(),"dock");
    fullScreenToggle = new KAction(i18n("Full-Screen Mode"),"window_fullscreen",SHIFT+CTRL+Key_F,this,SLOT(slotFullScreenToggle()),actionCollection(),"fullScreenToggle");
    //setHelpMenuEnabled(false);
    statusBar()->setSizeGripEnabled(false);
    setStandardToolBarMenuEnabled(true);

    tabDock = createDockWidget("tabDock",QPixmap());
    tabDock->setEnableDocking(KDockWidget::DockNone);
    tab = new TaviaTab(tabDock,"tab");
    tabDock->setWidget(tab);
    makeDockVisible(tabDock);
    setView(tabDock);
    setMainDockWidget(tabDock);

    sidebarDock = createDockWidget("sidebarDock",QPixmap(),0,i18n("Sidebar"));
    sidebar = new TaviaSidebar(sidebarDock,"sidebar");
    sidebar->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    sidebarDock->setWidget(sidebar);
    makeDockVisible(sidebarDock);
    sidebarDock->manualDock(tabDock,KDockWidget::DockLeft,20);

    tabboxDock = createDockWidget("tabBoxDock",QPixmap(),0,i18n("TabBox"));
    tabboxDock->setWidget(tab->getBBox());
    makeDockVisible(tabboxDock);
    tabboxDock->manualDock(tabDock,KDockWidget::DockTop,10);

    manager()->setSplitterKeepSize(true);

    connect(tab,SIGNAL(setStatusBarText(const QString &)),
        statusBar(),SLOT(message(const QString &)));
    createGUI("taviaui.rc");

    connect(sidebar,SIGNAL(openURL(const KURL &)),tab,SLOT(slotOpenURL(const KURL &)));
    connect(sidebar,SIGNAL(openURL(const KURL &,const KParts::URLArgs &)),tab,SLOT(slotOpenURL(const KURL &,const KParts::URLArgs &)));
    connect(sidebar,SIGNAL(openURLwithNewTab(const KURL &)),tab,SLOT(addView(const KURL &)));
    connect(sidebar,SIGNAL(openURLwithNewTab(const KURL &,const KParts::URLArgs &)),tab,SLOT(addView(const KURL &,const KParts::URLArgs &)));

    //beginXMLPlug(this);
    //guiFactory()->addClient(this);
    guiFactory()->addClient(tab);
    //activateDock();
    setAutoSaveSettings();

    KConfig* config = KGlobal::config();
    readDockConfig(config,"DockLayout");
    //config->setGroup("sidbarsplitter");
    //splitter->setSizes(config->readIntListEntry("sizes"));
    //splitter->setOpaqueResize(false);
    resizeReady = false;
    tab->addView(KURL("about:tavia"),KParts::URLArgs());

    emit sidebar->initSidebar();

}

tavia::~tavia()
{
    KConfig* config = KGlobal::config();
    writeDockConfig(config,"DockLayout");
    //config->writeEntry("sizes",splitter->sizes());
}

/*!
    \fn tavia::bugReport()
 */
void tavia::bugReport()
{
    emit tab->addView(KURL("http://nop.net-p.org/modules/newbb/viewforum.php?forum=6"),KParts::URLArgs());
}

/*!
    \fn tavia::slotFullScreenToggle()
 */
void tavia::slotFullScreenToggle()
{
    resizeReady = true;
    // fullscreenɡKonquerorΥɤ㤷ȡ
    if(isFullScreen()) {
        fullScreenToggle->setText(i18n("Full-Screen Mode"));
        fullScreenToggle->setIcon("window_fullscreen");
        showNormal();
    } else {
        fullScreenToggle->setText(i18n("Exit Full-Screen Mode"));
        fullScreenToggle->setIcon("window_nofullscreen");
        showFullScreen();
    }
    resizeReady = false;
}


/*!
    \fn tavia::slotDockMenuPopup()
 */
void tavia::slotDockMenuPopup()
{
    dockHideShowMenu()->popup(QCursor::pos());
}

/*!
    \fn tavia::resize(int w,int h)
 */
void tavia::resize(int w,int h)
{
    // JavaScriptˤ륦ɥꥵ̵롣
    // κǤ롣¾ˡϤʤ
    if (resizeReady) QWidget::resize(w,h);
    return;
}


/*!
    \fn tavia::resize(const QSize &s)
 */
void tavia::resize(const QSize &s)
{
    QWidget::resize(s.width(),s.height());
}
