/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviafileio.h"
#include <kio/slaveconfig.h>
#include <kio/scheduler.h>

TaviaFileIO::TaviaFileIO(QObject *parent, const char *name)
 : QObject(parent, name)
{
    //jobs.setAutoDelete(true);
    //KIO::SlaveConfig::self()->setConfigData("http","","UserAgent","Tavia-FileIODebug");

}


TaviaFileIO::~TaviaFileIO()
{
}



/*!
    \fn TaviaFileIO::get(const KURL &url,const KParts::URLArgs &args)
 */
int TaviaFileIO::get(const KURL &url,const KParts::URLArgs &args)
{
    KIO::TransferJob * job;
    if (args.doPost() && url.protocol().startsWith("http")) {
        job = KIO::http_post(url,args.postData,false);
        job->addMetaData("content-type",args.contentType());
        //job->addMetaData("UserAgent","Tavia-FileIODebug");
    } else {
        job = KIO::get(url,args.reload,false);
        //job->addMetaData("UserAgent","Tavia-FileIODebug");
    }
    jobs.append(job);
    connect(job,SIGNAL(data(KIO::Job *, const QByteArray &)),
        this,SLOT(slotData(KIO::Job *, const QByteArray &)));
    connect(job,SIGNAL(mimetype(KIO::Job *, const QString &)),
        this,SLOT(slotMimeType(KIO::Job *, const QString &)));

    return(jobs.find(job));
}




/*!
    \fn TaviaFileIO::slotData(KIO::Job *job, const QByteArray &data)
 */
void TaviaFileIO::slotData(KIO::Job *job, const QByteArray &data)
{
    static_cast<KIO::TransferJob *>(job)->suspend();
    emit sendData(jobs.find(job),data);
    static_cast<KIO::TransferJob *>(job)->resume();
}


/*!
    \fn TaviaFileIO::slotMimeType(KIO::Job *job, const QString &type)
 */
void TaviaFileIO::slotMimeType(KIO::Job *job, const QString &type)
{
    emit sendMimeType(jobs.find(job),type);
}


/*!
    \fn TaviaFileIO::getURL(int job)
 */
KURL TaviaFileIO::getURL(int job)
{
    return(static_cast<KIO::TransferJob *>(jobs.at(job))->url());;
}


/*!
    \fn TaviaFileIO::mimetype(const KURL &url,const KParts::URLArgs &args)
 */
int TaviaFileIO::mimetype(const KURL &url,const KParts::URLArgs &args)
{
    KIO::TransferJob * job;
    job = KIO::mimetype(url,false);
    //job->addMetaData("UserAgent","Tavia-FileIODebug");
    jobs.append(job);
    connect(job,SIGNAL(mimetype(KIO::Job *, const QString &)),
        this,SLOT(slotMimeType(KIO::Job *, const QString &)));

    return(jobs.find(job));
}
