/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TaviaHTMLView_H
#define TaviaHTMLView_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qpainter.h>
#include <qmap.h>
#include <qcstring.h>
#include <qcursor.h>
#include <qpopupmenu.h>
#include <kapplication.h>
#include <khtml_part.h>
#include <khtmlview.h>
#include <klineedit.h>
#include <kxmlguiclient.h>
#include <kxmlguifactory.h>
#include <klocale.h>
#include <kaboutdata.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kparts/browserextension.h>
#include <kparts/factory.h>
#include <kparts/partmanager.h>
#include <kstdaction.h>
#include <kaction.h>
#include <dom/html_document.h>
#include <kio/job.h>
#include <kmimetype.h>

#include "taviahtmlcacheentry.h"
#include "taviafileio.h"
#include "taviarun.h"


/**
@author TAM(Teppei Tamra)
*/
class TaviaRun;

class TaviaHTMLView : public KHTMLPart
{
Q_OBJECT
public:
    TaviaHTMLView(KURL url = KURL(), KParts::URLArgs args = KParts::URLArgs(), QWidget *parent = 0, const char *name = 0, WFlags f = 0);

    ~TaviaHTMLView();
    friend class TaviaHTMLPopup;

protected:
    unsigned int currentHistory;
    QPtrList<TaviaHTMLCacheEntry> urlHistory;
    bool reloadHistory;
    bool noHistoryUpdate;
    bool backAction;
    bool forwardAction;
    bool reloadAction;
    bool stopAction;
    int thumbnailUpdateTimer;
    QPixmap thumbnailPixmap;
    QPixmap progressPixmap;
    TaviaHTMLPopup *popup;
    bool locked;
    TaviaFileIO *fileIO;
    int jobnum;
    TaviaRun *run;

private:
    bool completed;
    bool noCache;
    int pProgress;
    KParts::URLArgs pArgs;
    KURL pURL;

public:
    unsigned int getHistoryCount();
    unsigned int getCurrentHistory();
    QString getPageName();
    bool isLocked();
    void aboutTavia();
    void aboutGPL();
    void outputImage(const KURL &url);
    const QPtrList<TaviaHTMLCacheEntry> getHistoryList();

protected:
    void createThumbnail();

public slots:
    void slotOpenURL(const KURL &url,const KParts::URLArgs &args);
    void restoreHistory(unsigned int num);
    void reload();
    void slotStop();
    void slotStopRun();
    void slotBack();
    void slotForward();
    void slotReload();
    void slotBack(int num);
    void slotForward(int num);
    void sendActionStatus(const KURL &url = KURL());
    void setLock(bool l);

protected slots:
    void popupMenu(const QString &linkurl);
    void addHistory();
    void historyUpdate();
    void tSetURLArgs(const KURL &url, const KParts::URLArgs &args = KParts::URLArgs());
    void slotSetStatusBarText(const QString& text);
    void setSlaveConfig();
    void partAdded (KParts::Part *part);
    void partRemoved (KParts::Part *part);
    void slotMimeType(int job, const QString type);
    void slotData(int job,const QByteArray data);
    void progress(int p);

private slots:
    void slotCompleted();
    void slotCompleted(bool p);

signals:
    void thumbnailReady(const QPixmap &pixmap);
    void pageNameReady(const QString &name);
    void pageURLReady(TaviaHTMLView *t,const KURL &url);
    void startOpenURL();
    void completedUrl(TaviaHTMLView *t,const KURL  &url);
    void backActionEnabled(TaviaHTMLView *t,bool b);
    void forwardActionEnabled(TaviaHTMLView *t,bool b);
    void reloadActionEnabled(TaviaHTMLView *t,bool b);
    void stopActionEnabled(TaviaHTMLView *t,bool b);
    void viewLockEnabled(TaviaHTMLView *t,bool b);
    void sendStatusBarText(TaviaHTMLView *t,const QString& text);

};

#endif
