/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviaqnavi.h"
#include <qbitmap.h>
#include <qpainter.h>
#include <qimage.h>
#include <qcursor.h>
#include <qlabel.h>
#include <qlayout.h>
#include <kglobal.h>


TaviaQNavi::TaviaQNavi(QWidget *parent, TaviaTab *taviatab, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
    busy = true;
    tab = taviatab;
    dragMove = false;
    nataFrame.load(locate("data","tavia/nata.png"));
    createThumbnails();
    setAutoMask(true);
    setMask(QBitmap(locate("data","tavia/natamask.xbm")));
    setMouseTracking(true);
    setCursor(Qt::BlankCursor);
    grabMouse();
    grabKeyboard();
    //grabMouse();
    defPoint = QCursor::pos();
    move(defPoint.x() - 50,defPoint.y() - 50);
    setFixedSize(100,100);
    setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    //resize(100,100);
    show();

    changeThumbnail();

    busy = false;
}


TaviaQNavi::~TaviaQNavi()
{
    releaseMouse();
    releaseKeyboard();
    // 󡼡
    while(busy) {}
    return;
}



/*!
    \fn TaviaQNavi::mousePressEvent(QMouseEvent * e)
 */
void TaviaQNavi::mousePressEvent(QMouseEvent * e)
{
    if (e->button() == Qt::LeftButton) {
        dragMove = true;
    }
}



/*!
    \fn TaviaQNavi::mouseMoveEvent(QMouseEvent * e)
 */
void TaviaQNavi::mouseMoveEvent(QMouseEvent * e)
{
    if (dragMove) {
        defPoint.setX(e->globalX());
        defPoint.setY(e->globalY());
        move(defPoint.x() - 50,defPoint.y() - 50);
        return;
    }
    int x,y;
    x = e->globalX() - defPoint.x();
    y = e->globalY() - defPoint.y();
    QCursor::setPos(defPoint);
    tab->htmlList.at(tab->getSelected())->view()->scrollBy(x,y);
}


/*!
    \fn TaviaQNavi::wheelEvent(QWheelEvent * e)
 */
void TaviaQNavi::wheelEvent(QWheelEvent * e)
{
    if (e->delta() >= 0) emit tab->prevTab();
    else emit tab->nextTab();
    changeThumbnail();
}


/*!
    \fn TaviaQNavi::mouseReleaseEvent(QMouseEvent * e)
 */
void TaviaQNavi::mouseReleaseEvent(QMouseEvent * e)
{
    if (e->button() == Qt::LeftButton) dragMove = false;
    if (e->button() == Qt::RightButton) emit qnaviClose();
}




/*!
    \fn TaviaQNavi::createThumbnails()
 */
void TaviaQNavi::createThumbnails()
{
    for(unsigned int i = 0;i < tab->htmlList.count();i ++) {
        QPixmap pix;
        QPainter p;
        pix.resize(400,400);
        QRect rc(0,40,400,320);
        p.begin(&pix);
        tab->htmlList.at(i)->paint(&p, rc);
        p.end();
        QImage img = pix.convertToImage();
        pix.convertFromImage(img.smoothScale(100,100));
        bitBlt(&pix,0,0,&nataFrame);
        thumbnails.append(pix);
    }
}


/*!
    \fn TaviaQNavi::changeThumbnail()
 */
void TaviaQNavi::changeThumbnail()
{
    bitBlt(this,0,0,&thumbnails[tab->getSelected()]);
}
