/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TAVIATAB_H
#define TAVIATAB_H

#include <qwidget.h>
#include <qptrlist.h>
#include <qlayout.h>
#include <tsessview.h>
#include <taviahtmlview.h>
#include <kxmlguiclient.h>
#include <kxmlguifactory.h>
#include <khtml_part.h>
#include <khtmlview.h>
#include <kurl.h>
#include <klocale.h>
#include <kaboutdata.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kparts/browserextension.h>
#include <kparts/factory.h>
#include <kstdaction.h>
#include <kaction.h>
#include <kurlcombobox.h>
#include <kbookmarkmenu.h>
#include <kbookmarkmanager.h>
#include <taviaurlbar.h>


/**
@author TAM(Teppei Tamra)
*/
class TaviaTab : public TSESSView, public KXMLGUIClient, public KBookmarkOwner
{
Q_OBJECT
public:
    TaviaTab(QWidget *parent = 0, const char *name = 0);

    ~TaviaTab();
    friend class TaviaQNavi;

public:
    TaviaHTMLView * getView(int num);
    virtual void openBookmarkURL(const QString &url);
    virtual QString currentTitle() const;
    virtual QString currentURL() const;

protected:
    bool isCurrentView(TaviaHTMLView *t);

protected:
    KAction * backAction;
    KAction * forwardAction;
    KAction * reloadAction;
    KAction * stopAction;
    KActionMenu * bookmarkMenuAction;
    KBookmarkMenu * bookmarkMenu;
    KToggleAction * viewLock;
    KWidgetAction * urlBar;
    TaviaURLBar * uCombo;
    KAction * urlFocus;
    QPtrList<TaviaHTMLView> htmlList;
    TaviaHTMLView * currentGUI;
    KBookmarkManager *bookmarkManager;
    TaviaQNavi * navi;

public slots:
    void delView(int num);
    int addView(const KURL &url);
    int addView(const KURL &url,const KParts::URLArgs &args);
    int addView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p);
    int addView();
    void newTab();
    void rmTab();
    void prevTab();
    void nextTab();
    void slotOpenURL(const KURL &url);
    void slotOpenURL(const KURL &url,const KParts::URLArgs &args);

protected slots:
    void pageBack();
    void pageForward();
    void pageReload();
    void pageStop();
    void backActionEnabled(TaviaHTMLView *t,bool b);
    void forwardActionEnabled(TaviaHTMLView *t,bool b);
    void reloadActionEnabled(TaviaHTMLView *t,bool b);
    void stopActionEnabled(TaviaHTMLView *t,bool b);
    void viewLockEnabled(TaviaHTMLView *t,bool b);
    void guiSetting(int num);
    void dropURLList(int num,const KURL::List list);
    void addURLBarHistory(TaviaHTMLView *t,const KURL &url);
    void updateURLBar(TaviaHTMLView *t,const KURL &url);
    void setStatusBarText(TaviaHTMLView *t,const QString &text);
    void setViewLock(bool l);

signals:
    void setStatusBarText(const QString &text);




protected slots:
    void qnavi();
    void qnaviClose();
protected slots:
    void destroyedView(QObject *t);
};

#endif
