/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviaurlbar.h"
#include <kcombobox.h>

TaviaURLBar::TaviaURLBar(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
    uCombo = new KURLComboBox(KURLComboBox::Both,this,"urlCombo");
    uCombo->setEditable(true);
    uCombo->setAutoCompletion(true);
    uCombo->setContextMenuEnabled(true);
    uCombo->setURLDropsEnabled(true);
    uCombo->setInsertionPolicy(QComboBox::NoInsertion);
    uCombo->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);

    urlEnter = new QToolButton(this,"urlEnter");
    urlEnter->setAutoRaise(true);
    urlEnter->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("key_enter",KIcon::Toolbar,KIcon::SizeSmall));

    urlErase = new QToolButton(this,"urlErase");
    urlErase->setAutoRaise(true);
    urlErase->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("locationbar_erase",KIcon::Toolbar,KIcon::SizeSmall));

    QLabel *label = new QLabel(this,"label");
    label->setText(i18n("URL:"));
    QHBoxLayout *lay = new QHBoxLayout(this);
    lay->addWidget(urlErase);
    lay->addWidget(label);
    lay->addWidget(uCombo);
    lay->addWidget(urlEnter);

    connect(urlErase,SIGNAL(clicked()),this,SLOT(clearURLBar()));
    connect(urlEnter,SIGNAL(clicked()),this,SLOT(returnPressed()));
    connect(uCombo,SIGNAL(urlActivated(const KURL &)),this,SLOT(slotEnterURL(const KURL &)));
    connect(uCombo,SIGNAL(returnPressed()),this,SLOT(returnPressed()));
}


TaviaURLBar::~TaviaURLBar()
{
}

/*!
    \fn TaviaURLBar::setFocus()
 */
void TaviaURLBar::setFocus()
{
    uCombo->setFocus();
}

/*!
    \fn TaviaURLBar::clearURLBar()
 */
void TaviaURLBar::clearURLBar()
{
    uCombo->setFocus();
    uCombo->setEditURL("");
}

/*!
    \fn TaviaURLBar::returnPressed()
 */
void TaviaURLBar::returnPressed()
{
    emit enterURL(KURL(uCombo->currentText()));
}

/*!
    \fn TaviaURLBar::setEditURL(const KURL &url)
 */
void TaviaURLBar::setEditURL(const KURL &url)
{
    uCombo->setEditURL(url);
}

/*!
    \fn TaviaURLBar::addURLHistory(const KURL &url)
 */
void TaviaURLBar::addURLHistory(const KURL &url)
{
    // ʣɤ
    if( uCombo->contains(url.url()) ) {
        return;
    }
    uCombo->insertURL(url);
    uCombo->setCurrentItem(uCombo->count() - 1);
    compList.append(url.url());
    //completionBox()->setItem(compList);
}

/*!
    \fn TaviaURLBar::slotEnterURL(const KURL &url)
 */
void TaviaURLBar::slotEnterURL(const KURL &url)
{
    enterURL(url);
}

/*!
    \fn TaviaURLBar::setEnabled(bool b)
 */
void TaviaURLBar::setEnabled(bool b)
{
    // ޤ̵̣
    if (!b) {
        uCombo->clearFocus();
    }
    QWidget::setEnabled(b);
}






#include "taviaurlbar.moc"
